local M = {}

local helper = require('scenario/scenariohelper')
local rotation = {["d_I"] = 0, ["d_J"] = 0, ["b_I"] = 0, ["b_J"] = 0, ["da_I"] = 0, ["da_J"] = 0}
local active = {["divider"] = false, ["divider2"] = true, ["divider3"] = false, ["dividerA1"] = false, ["dividerA2"] = false}
local reset = 1

local function onScenarioChange(scenario)
	if scenario.state == 'pre-start' then
		active = {["divider"] = false, ["divider2"] = true, ["divider3"] = false, ["dividerA1"] = false, ["dividerA2"] = false}
		rotation = {["d_I"] = 0, ["d_J"] = 0, ["b_I"] = 0, ["b_J"] = 0, ["da_I"] = 0, ["da_J"] = 0}
		reset = 1
	end
end

local function onRaceWaypointReached(data)
	if data.vehicleName == 'scenario_player0' then
		if data.waypointName == 'WayPoint1' then
			helper.flashUiMessage('This wire is connected so this divider is powered', 5) 
		elseif data.waypointName == 'WayPoint2' then
			helper.flashUiMessage('This wire is not connected...', 5) 
		elseif data.waypointName == 'WayPoint3' then
			helper.flashUiMessage('This is an advanced divider; you need 2 power sources to power it', 5)  
		elseif data.waypointName == 'WayPoint4' then
			helper.flashUiMessage('A basic divider will overload if 2 power sources are connected...', 5)  
		end
	end
end


local function onBeamNGTrigger(data)
	if data.triggerName == 'circuitBreaker1' then
		if not active["divider"] then
			TorqueScript.eval([[
				circuit1Unpowered.position = "10000 0 0";
				circuit1Powered.position = "145.945 23.3926 39.086";
				door1.position = "10000 0 0";
			]])
			active["divider"] = true
		else
			TorqueScript.eval([[
				circuit1Powered.position = "10000 0 0";
				circuit1Unpowered.position = "145.945 23.3926 39.086";
				door1.position = "370.077 118.436 -0.184723";
			]])
			active["divider"] = false
		end
		be:reloadStaticCollision();
	elseif data.triggerName == 'circuitBreaker2' then
		if not active["dividerA1"] then
			TorqueScript.eval([[
				circuit2Unpowered.position = "10000 0 0";
				circuit2Powered.position = "366.061 62.6028 65.0794";
				door2.position = "10000 0 0";
			]])
			active["dividerA1"] = true
		else
			TorqueScript.eval([[
				circuit2Powered.position = "10000 0 0";
				circuit2Unpowered.position = "366.061 62.6028 65.0794";
				door2.position = "273.583 19.4359 -0.184723";
			]])
			active["dividerA1"] = false
		end
		be:reloadStaticCollision();
	elseif data.triggerName == 'circuitBreaker3' then
		if not active["divider2"] then
			TorqueScript.eval([[
				circuit3Powered1.position = "10000 0 0";
				circuit3Unpowered1.position = "246.072 -71.036 18.9816";
			]])
			active["divider2"] = true
		else
			TorqueScript.eval([[
				circuit3Unpowered1.position = "10000 0 0";
				circuit3Powered1.position = "246.072 -71.036 18.9816";
			]])
			active["divider2"] = false
		end
	elseif data.triggerName == 'circuitBreaker4' then
		if not active["divider3"] then
			TorqueScript.eval([[
				circuit3Unpowered2.position = "10000 0 0";
				circuit3Powered2.position = "245.924 -20.7992 18.696";
			]])
			active["divider3"] = true
		else
			TorqueScript.eval([[
				circuit3Powered2.position = "10000 0 0";
				circuit3Unpowered2.position = "245.924 -20.7992 18.696";
			]])
			active["divider3"] = false
		end
	elseif data.triggerName == 'circuitBreaker5' then
		if not active["dividerA2"] then
			TorqueScript.eval([[
				circuit3Unpowered3.position = "10000 0 0";
				circuit3Powered3.position = "171.926 -20.8251 18.696";
				pipe.position = "0.705545 0.0664334 0.705545 172.398";
			]])
			active["dividerA2"] = true
		else
			TorqueScript.eval([[
				circuit3Powered3.position = "10000 0 0";
				circuit3Unpowered3.position = "171.926 -20.8251 18.696";
				pipe.position = "10000 0 0";
			]])
			active["dividerA2"] = false
		end
	end
end

local function rotateObjects()
	--Update rotations
	--Divider
	rotation["d_I"] = rotation["d_I"]+4
	rotation["d_J"] = rotation["d_J"]+6
	rotation["da_I"] = rotation["da_I"]+8
	rotation["da_J"] = rotation["da_J"]+12
	--Battery - all should be on constantly
	rotation["b_I"] = rotation["b_I"]+5
	rotation["b_J"] = rotation["b_J"]+10
	
	TorqueScript.eval([[
		batteryDisc1.rotation =    "0 ]]..rotation["b_I"]..[[ ]]..rotation["b_I"]..[[ ]]..rotation["b_I"]..[[";
		batteryDisc2.rotation =    "0 ]]..rotation["b_J"]..[[ ]]..rotation["b_J"]..[[ ]]..rotation["b_J"]..[[";
		batteryDisc3.rotation =    "0 ]]..rotation["b_I"]..[[ ]]..rotation["b_I"]..[[ ]]..rotation["b_I"]..[[";
		batteryDisc4.rotation =    "0 ]]..rotation["b_J"]..[[ ]]..rotation["b_J"]..[[ ]]..rotation["b_J"]..[[";
		batteryDisc5.rotation =    "0 ]]..rotation["b_I"]..[[ ]]..rotation["b_I"]..[[ ]]..rotation["b_I"]..[[";
		batteryDisc6.rotation =    "0 ]]..rotation["b_J"]..[[ ]]..rotation["b_J"]..[[ ]]..rotation["b_J"]..[[";
		batteryDisc7.rotation =    "0 ]]..rotation["b_I"]..[[ ]]..rotation["b_I"]..[[ ]]..rotation["b_I"]..[[";
		batteryDisc8.rotation =    "0 ]]..rotation["b_J"]..[[ ]]..rotation["b_J"]..[[ ]]..rotation["b_J"]..[[";
		batteryDisc9.rotation =    "0 ]]..rotation["b_I"]..[[ ]]..rotation["b_I"]..[[ ]]..rotation["b_I"]..[[";
		batteryDisc10.rotation =   "0 ]]..rotation["b_J"]..[[ ]]..rotation["b_J"]..[[ ]]..rotation["b_J"]..[[";
		batteryDisc11.rotation =   "0 ]]..rotation["b_I"]..[[ ]]..rotation["b_I"]..[[ ]]..rotation["b_I"]..[[";
		batteryDisc12.rotation =   "0 ]]..rotation["b_J"]..[[ ]]..rotation["b_J"]..[[ ]]..rotation["b_J"]..[[";

		dividerDisc1_1.rotation =  "0 ]]..rotation["d_I"]..[[ ]]..rotation["d_I"]..[[ ]]..rotation["d_I"]..[[";
		dividerDisc1_2.rotation =  "0 ]]..rotation["d_J"]..[[ ]]..rotation["d_J"]..[[ ]]..rotation["d_J"]..[[";
		
		dividerDisc3_3.rotation =  "0 ]]..rotation["d_I"]..[[ ]]..rotation["d_I"]..[[ ]]..rotation["d_I"]..[[";
		dividerDisc3_4.rotation =  "0 ]]..rotation["d_J"]..[[ ]]..rotation["d_J"]..[[ ]]..rotation["d_J"]..[[";
		dividerDisc3_9.rotation =  "0 ]]..rotation["d_I"]..[[ ]]..rotation["d_I"]..[[ ]]..rotation["d_I"]..[[";
		dividerDisc3_10.rotation = "0 ]]..rotation["d_J"]..[[ ]]..rotation["d_J"]..[[ ]]..rotation["d_J"]..[[";
		dividerDisc3_11.rotation = "0 ]]..rotation["d_I"]..[[ ]]..rotation["d_I"]..[[ ]]..rotation["d_I"]..[[";
		dividerDisc3_12.rotation = "0 ]]..rotation["d_J"]..[[ ]]..rotation["d_J"]..[[ ]]..rotation["d_J"]..[[";
	]])
	
	if active["divider"] then
		TorqueScript.eval([[
			dividerDisc1_3.rotation = "0 ]]..rotation["d_I"]..[[ ]]..rotation["d_I"]..[[ ]]..rotation["d_I"]..[[";
			dividerDisc1_4.rotation = "0 ]]..rotation["d_J"]..[[ ]]..rotation["d_J"]..[[ ]]..rotation["d_J"]..[[";
		]])
	end
	if active["divider2"] then
		TorqueScript.eval([[
			dividerDisc3_7.rotation =  "0 ]]..rotation["d_I"]..[[ ]]..rotation["d_I"]..[[ ]]..rotation["d_I"]..[[";
			dividerDisc3_8.rotation =  "0 ]]..rotation["d_J"]..[[ ]]..rotation["d_J"]..[[ ]]..rotation["d_J"]..[[";
		]])
	end
	if active["divider3"] then
		TorqueScript.eval([[
			dividerDisc3_13.rotation = "0 ]]..rotation["d_I"]..[[ ]]..rotation["d_I"]..[[ ]]..rotation["d_I"]..[[";
			dividerDisc3_14.rotation = "0 ]]..rotation["d_J"]..[[ ]]..rotation["d_J"]..[[ ]]..rotation["d_J"]..[[";
		]])
	end
	if active["dividerA1"] then
		TorqueScript.eval([[
			dividerDiscA1.rotation = "0 ]]..rotation["da_I"]..[[ ]]..rotation["da_I"]..[[ ]]..rotation["da_I"]..[[";
			dividerDiscA2.rotation = "0 ]]..rotation["da_J"]..[[ ]]..rotation["da_J"]..[[ ]]..rotation["da_J"]..[[";
		]])
	end
	if active["dividerA2"] then
		TorqueScript.eval([[
			dividerDisc3_5.rotation = "0 ]]..rotation["d_I"]..[[ ]]..rotation["d_I"]..[[ ]]..rotation["d_I"]..[[";
			dividerDisc3_6.rotation = "0 ]]..rotation["d_J"]..[[ ]]..rotation["d_J"]..[[ ]]..rotation["d_J"]..[[";
			
			dividerDiscA3.rotation = "0 ]]..rotation["da_I"]..[[ ]]..rotation["da_I"]..[[ ]]..rotation["da_I"]..[[";
			dividerDiscA4.rotation = "0 ]]..rotation["da_J"]..[[ ]]..rotation["da_J"]..[[ ]]..rotation["da_J"]..[[";
		]])
	end
end

local function onPreRender(dt)
	if reset ~= 0 and reset < 5 then
		TorqueScript.eval([[
			door1.position = "181.139 116.034 -0.184723";
			door2.position = "273.583 19.4359 -0.184723";
			pipe.position = "0.705545 0.0664334 0.705545 172.398";
		]])
		be:reloadStaticCollision();
		reset = reset + 1
	else reset = 0
	end

	rotateObjects()
end

M.onRaceWaypointReached = onRaceWaypointReached
M.onScenarioChange = onScenarioChange
M.onPreRender = onPreRender
M.onBeamNGTrigger = onBeamNGTrigger
return M